"""Set of constants."""


TYPE_KEY = "__type__"
DATA_KEY = "__data__"
INDEX_STORE_KEY = "index_store"
DOC_STORE_KEY = "doc_store"
DREAMS_ANALYSIS_STORE_KEY = "dreams_analysis_store"
TASK_STEP_STORE_KEY = "task_step_store"