"""

""" 

# 03- 故事情境生成 `story_scenario_context`.txt
STORY_BOARD_SCENE_TEMPLATE = """根据文本执行任务。以下任务信息

作为一个社会学研究学者，您已经查阅了《作为激情的爱情》卢曼编写的书籍，尝试通过参考文献中定义的爱情语义学，从文本中总结下方片段
在如下表征信息中，研究其突出特点
研究交流媒介领域的语义信息
研究激情的非理性与风雅情术的偶然性
研究自身的快感是否转移到社会行为上
研究语义信息的固定形式与预期落空因果性，是否存在可激发性拓展到否定物之中
主要突出如下几点
1、我们建议研究社交媒体中的爱情表现。
2、我们提议研究卢曼的爱情语义学在电视和电影中的应用。
3、我们推荐深入研究卢曼的理论，以了解其在社会学研究中的潜在应用。
4、这个研究与社会学研究中的情感因素密切相关。
5、我们的讨论与媒体中的语义信息有关。
6、卢曼的爱情语义学与社交媒体的内容密切相联。
7、我们需要考虑卢曼理论和社会学研究的靠近度。
8、了解卢曼理论如何接近社交媒体中的爱情表现。
9、这个话题与卢曼的爱情语义学及其在社会学中的附近研究领域有关。
10、我们可以探讨卢曼理论和社会学的交叉点。
11、社交媒体中的爱情表现与卢曼理论存在重叠。
12、在这个研究中，卢曼的爱情语义学与社会学观点相交。
你可以尝试分步思考然后告诉我答案，Step by Step Decomposition



文本内容如下

{scene_content}
"""

# 03-故事场景生成 `scene_monologue_context`.txt
STORY_BOARD_SUMMARY_CONTEXT_TEMPLATE = """根据下面人物对话分析这里关于{user_id}今天发生了什么，编写一个独白信息，使用第一人称描述。


人物对话如下

{story_board_summary_context}
"""


# 05-剧情总结 `evolutionary_step`.txt
EDREAMS_EVOLUTIONARY_TEMPLATE = """作为一个社会学研究学者，尝试通过社会学研究相关内容总结下方的对话
社会学研究相关内容分成如下步骤
研究交流媒介领域的语义信息
研究激情的非理性与风雅情术的偶然性
研究自身的快感是否转移到社会行为上
研究语义信息的固定形式与预期落空因果性，是否存在可激发性拓展到否定物之中
你可以尝试分步思考然后告诉我答案，Step by Step Decomposition



人物对话如下

{story_board_summary_context}
"""

# 05-性格分析 `dreams_personality_context`.txt
EDREAMS_PERSONALITY_TEMPLATE = """作为一个社会学研究学者，您已经查阅了《作为激情的爱情》卢曼编写的书籍，尝试通过参考文献中定义的爱情语义学总结下方片段
社会学研究相关内容分成如下步骤
研究交流媒介领域的语义信息
研究激情的非理性与风雅情术的偶然性
研究自身的快感是否转移到社会行为上
研究语义信息的固定形式与预期落空因果性，是否存在可激发性拓展到否定物之中

根据上述信息,总结片段中人物的性格
你可以尝试分步思考然后告诉我答案，Step by Step Decomposition



{evolutionary_step}
"""


# 04-情感情景引导.txt
DREAMS_GEN_TEMPLATE = """作为一个心理咨询工作者，尝试使用开放性问题与来访者交流，尝试带入他们的感受和思维，下面内容中已经整理了关于{user_id}的`故事梗概`，`故事情境`,`故事场景`，请告诉我如何引导来访者说出自己的问题
你可以尝试分步思考然后告诉我答案，Step by Step Decomposition

例如
**Step 1: 建立情感连接**
开始时，您可以通过表达理解和共鸣来建立情感连接，让来访者感到舒适。您可以说：“我注意到这个对话中有许多愉快的时刻和互动。你对这些时刻有什么特别的感受吗？”

**Step 2: 探索来访者的感受**
继续引导来访者表达他们的感受。您可以问：“在这个对话中，有哪些瞬间让你感到开心或快乐？”

**Step 3: 询问是否有反感情绪**
除了积极的情感，也要问询是否有一些负面情感或担忧。您可以说：“除了快乐的瞬间，是否有一些让你感到不安或担忧的地方？”



# 故事梗概
 
{story_board_summary_context}


# 故事情境

{story_scenario_context}


# 故事场景

{scene_monologue_context}
"""

